package io.mockk.it

import io.mockk.every
import io.mockk.mockk
import kotlin.test.Test
import kotlin.test.Ignore
import kotlin.test.assertEquals


class SealedInterfaceTest {

    @Test
    @Ignore("Fails on JDK17+ https://github.com/mockk/mockk/issues/832")
    fun serviceReturnsSealedClassImpl() {
        val factory = mockk<Factory> {
            every { create() } returns Leaf(1)
        }

        val result = factory.create()

        assertEquals(Leaf(1), result)
    }

    @Test
    @Ignore("Fails on JDK17+ https://github.com/mockk/mockk/issues/832")
    fun serviceAnswersSealedClassImpl() {
        val factory = mockk<Factory> {
            every { create() } answers { Leaf(1) }
        }

        val result = factory.create()

        assertEquals(Leaf(1), result)
    }

    companion object {

        sealed interface Node

        data class Root(val id: Int) : Node
        data class Leaf(val id: Int) : Node

        interface Factory {
            fun create(): Node
        }

        class FactoryImpl : Factory {
            override fun create(): Node = Root(0)
        }

    }
}
